// (c) Max Kresch, licensed under GPLv3, 2008.
//     See LICENSE for further details.

#ifndef MATRIX_H
#define MATRIX_H

#include "Vector.H"

class Matrix {
  public:
    inline Matrix(void):N(0),elts(new double[0]) { }
    inline Matrix(const int N): N(N),elts(new double[N*N]) { }
    inline Matrix(const Matrix& M): N(M.N),elts(new double[M.N*M.N]) {
      for(int i=0;i<N*N;i++){ elts[i] = M.elts[i]; }
    }
    inline ~Matrix(void){ delete[] elts; }
    inline Matrix operator=(const Matrix& M){ 
      N = M.N;
      delete[] elts;
      elts = new double[N*N];
      for(int i=0;i<N*N;i++){ elts[i] = M.elts[i]; }
      return *this;
    }
    inline void dump(void) const{   
      for(int r=0;r<N;r++){
        printf("| ");
        for(int c=0;c<N;c++){ printf("% 7.3f ",elts[r*N+c]); }
        printf("|\n");
      } 
    }
    inline Matrix operator*(const double d) const{ 
      Matrix res(N);
      for(int i=0;i<N*N;i++){ res.elts[i] = elts[i]*d; }
      return res;
    }
    inline Vector operator*(const Vector& V) const{ 
      Vector res(N);
      for(int i=0;i<N;i++){ res.elts[i] = 0.0; }
      for(int r=0;r<N;r++){
        for(int c=0;c<N;c++){
          res.elts[r] += elts[r*N+c]*V.elts[c];
        }
      }
      return res;
    }
    inline void toText(const char* filename) const{ 
      FILE* io=fopen(filename,"wc");
      for(int r=0;r<N;r++){
        for(int c=0;c<N;c++){ fprintf(io,"%lf ",elts[r*N+c]); }
        fprintf(io,"\n");
      }
      fclose(io);
    }
  public:
    int N;
    double* elts;
};

#endif // MATRIX_H
