// (c) Max Kresch, licensed under GPLv3, 2008.
//     See LICENSE for further details.

#ifndef VECTOR_H
#define VECTOR_H

#include <vector>
#include <string.h>
#include <stdlib.h>

class Vector {
  public:
    inline Vector(void):N(0),elts(new double[0]) { }
    inline Vector(const Vector& V): N(V.N),elts(new double[V.N]) {
      for(int i=0;i<N;i++){ elts[i] = V.elts[i]; }
    }
    inline ~Vector(void){ delete[] elts; }
    inline Vector(const int N): N(N),elts(new double[N]) { }
    inline Vector operator=(const Vector& V){ 
      N = V.N;
      delete[] elts;
      elts =new double[N];
      for(int i=0;i<N;i++){ elts[i]=V.elts[i]; }
      return *this;
    }
    inline void normalize(void){  
      double sum(0);
      for(int i=0;i<N;i++){ sum += elts[i]; }
      for(int i=0;i<N;i++){ elts[i] /= sum; }
    }
    inline void normalize(const double d){  
      double sum(0);
      for(int i=0;i<N;i++){ sum += elts[i]; }
      for(int i=0;i<N;i++){ elts[i] *= d/sum; }
    }
    inline Vector operator*(double d){ 
      Vector res(N);
      for(int i=0;i<N;i++){ res.elts[i]= d*elts[i];}
      return res;
    }
    inline void dumpR(void) const{   
      printf("<");
      for(int i=0;i<N-1;i++){printf("% 12.8f ",elts[i]);}
      printf("% 10.8e",elts[N-1]);
      printf(">\n");
    }
    inline void dump(void) const{   
      for(int i=0;i<N;i++){printf("|% 7.3f |\n",elts[i]);}
    }
    inline void toText(const char * filename){
     FILE* io=fopen(filename,"wc");
     for(int i=0;i<N;i++){ fprintf(io,"%lf\n",elts[i]); }
     fclose(io);
    }
    inline void fromText(const char * filename){
      std::vector<double> tmpElts;
      char *tok = NULL;
      char curr_line[100];
      char delim[] = " ";
      FILE* io=fopen(filename,"r");
      while(fgets(curr_line,100,io)){
        tmpElts.push_back( atof( strtok(curr_line,delim) ));
      }
      fclose(io);
      N = tmpElts.size();
      delete[] elts;
      elts = new double[N];
      for(int i=0;i<N;i++){ elts[i]=tmpElts[i]; }
      return;
    }
  public:
    int N;
    double* elts;
};

#endif // VECTOR_H
