// (c) Max Kresch, licensed under GPLv3, 2008.
//     See LICENSE for further details.

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "Vector.H"
#include "Matrix.H"
#include "Dos.H"
#include "Functs.H"

int main() {
  setvbuf(stdout,NULL,_IONBF,0);

  Dos DL;
  Dos DT;
  Dos Dsource;
  Dos Dtarget;

//--- Fast Changing User Input ------------------------------------------------
  double maxErr = 1e-6;
  double minStep = 1e-6;

  srand48(getpid());
  double QlStart( drand48()*1000.0 );
  double ClStart( drand48() );
  double QtStart( drand48()*1000.0);
  double CtStart( drand48() );

  double QLstep(10.0000);
  double CLstep(  0.1000);
  double QTstep(10.0000);
  double CTstep(  0.1000);

//--- Slow Changing User Input ------------------------------------------------

  DL.fromText("tests/data/ni_300.long");
  DT.fromText("tests/data/ni_300.tran");

  Dsource.fromText("tests/data/ni_300");
  Dtarget.fromText("tests/data/ni_575");

//--- End User Input ----------------------------------------------------------

// N = number of varied parameters, inserted `by hand` below.
// The last slot is for the error.
  int N = 4+1; 
  Vector QCE(N);
  QCE.elts[0] = QlStart;
  QCE.elts[1] = ClStart;
  QCE.elts[2] = QtStart;
  QCE.elts[3] = CtStart;
  QCE.elts[4] = 10000.0;

  Vector step(N);
  step.elts[0] = QLstep;
  step.elts[1] = CLstep;
  step.elts[2] = QTstep;
  step.elts[3] = CTstep;
  step.elts[4] = 0.0;

  DL.normalize();
  DT.normalize();
  Dsource.normalize();
  Dtarget.normalize();

  Vector W(Dtarget.N);
  for(int i=0;i<W.N;i++){
    if( Dtarget.y[i]>0.0 ){ W.elts[i]=1.0; } else { W.elts[i]=0.0; }
  }
  W.normalize();
  W.toText("tests/W");

  DL.toText("tests/DL");
  DT.toText("tests/DT");
  Dsource.toText("tests/Dsource");
  Dtarget.toText("tests/Dtarget");
  
  int sameErr = 0;

  while( QCE.elts[4] > maxErr && sameErr < 16 ){ //2^N == 16
    QCE.dumpR();
    for(int parity=-1;parity<2;parity+=2){
      for(int i=0;i<N;i++){
        sameErr += 1;
        Vector qce = QCE;
        if(qce.elts[i] + step.elts[i]*parity > 0.0 ){
          qce.elts[i] += step.elts[i]*parity;
          Dos dl=DL.rescale(qce.elts[1]).rebin(DL.x);
          dl = dl.broaden(qce.elts[0]);
          Dos dt=DT.rescale(qce.elts[3]).rebin(DT.x);
          dt = dt.broaden(qce.elts[2]);
          Dos D = dl + dt*2.0;
          D.normalize();
          double err = error(Dtarget,D,W);
          if( err < QCE.elts[4] ){
            sameErr = 0;
            QCE.elts[i] = qce.elts[i];
            QCE.elts[4] = err;
          }
        }
      }
    }
    if( sameErr >= 16 ){
      for(int i=0;i<N;i++){
        if(step.elts[i] > minStep){ step.elts[i] /= 10.0; sameErr = 0; }
      }
    }
  }
  QCE.toText("tests/QCE");
}
