// (c) Max Kresch, licensed under GPLv3, 2008.
//     See LICENSE for further details.

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "Vector.H"
#include "Matrix.H"
#include "Dos.H"
#include "Functs.H"

int main() {
  setvbuf(stdout,NULL,_IONBF,0);

  Dos DL;
  Dos DT;
  Dos Dsource;
  Dos Dtarget;

//--- Fast Changing User Input ------------------------------------------------
  double maxErr = 1e-6;
  double minStep = 1e-6;

  srand48(getpid());
  double QStart( drand48()*1000.0 );
  double CStart( drand48() );

  double Qstep(10.0000);
  double Cstep(  0.1000);

//--- Slow Changing User Input ------------------------------------------------

  Dsource.fromText("tests/data/ni_300");
  Dtarget.fromText("tests/data/ni_575");

//--- End User Input ----------------------------------------------------------

// N = number of varied parameters, inserted `by hand` below.
// The last slot is for the error.
  int N = 2+1; 
  Vector QCE(N);
  QCE.elts[0] = QStart;
  QCE.elts[1] = CStart;
  QCE.elts[2] = 10000.0;

  Vector step(N);
  step.elts[0] = Qstep;
  step.elts[1] = Cstep;
  step.elts[2] = 0.0;

  DL.normalize();
  DT.normalize();
  Dsource.normalize();
  Dtarget.normalize();

  Vector W(Dtarget.N);
  for(int i=0;i<W.N;i++){
    if( Dtarget.y[i]>0.0 ){ W.elts[i]=1.0; } else { W.elts[i]=0.0; }
  }
  W.normalize();
  W.toText("tests/W");

  Dsource.toText("tests/Dsource");
  Dtarget.toText("tests/Dtarget");
  
  int sameErr = 0;
  while( QCE.elts[N-1] > maxErr && sameErr < 16 ){ //2^N == 16
    QCE.dumpR();
    for(int parity=-1;parity<2;parity+=2){
      for(int i=0;i<N;i++){
        sameErr += 1;
        Vector qce = QCE;
        if(qce.elts[i] + step.elts[i]*parity > 0.0 ){
          qce.elts[i] += step.elts[i]*parity;
          Dos D=Dsource.rescale(qce.elts[1]).rebin(Dsource.x);
          D = D.broaden(qce.elts[0]);
          D.normalize();
          double err = error(Dtarget,D,W);
          if( err < QCE.elts[N-1] ){
            sameErr = 0;
            QCE.elts[i] = qce.elts[i];
            QCE.elts[N-1] = err;
          }
        }
      }
    }
    if( sameErr >= 16 ){
      for(int i=0;i<N;i++){
        if(step.elts[i] > minStep){ step.elts[i] /= 10.0; sameErr = 0; }
      }
    }
  }
  QCE.toText("tests/QCE");
}
